// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
import * as vscode from 'vscode';

// This method is called when your extension is activated
// Your extension is activated the very first time the command is executed
export function activate(context: vscode.ExtensionContext) {
	// Function to handle errors
	let handleError = (error: unknown) => { 
		if (error instanceof Error){ 
			vscode.window.showInformationMessage(`Test error: ${error.name} -> ${error.message}`);
		}
		else { 
			vscode.window.showInformationMessage(`Test error: Unknown error`); 
		}
	};


	let ctrl: vscode.TestController; 
		const rqst = new vscode.TestRunRequest();

		try {
			ctrl = vscode.tests.createTestController('test-api-functional-test.controllerId', 'AllTestAPITestController');
			console.log(`SUCCESS TestController created: ${ctrl.label}`);
		} catch (error) {
			console.log(`FAIL Could not create TestController`);
			handleError(error);
			throw new Error("Tests failed");
		}


	// Tests 
	let commandCreateController = vscode.commands.registerCommand('test-api-functional-test.runAllTest', () => {
		
		try { 
			const token = new vscode.CancellationTokenSource().token;
			const profile = ctrl.createRunProfile("myRunProfile", vscode.TestRunProfileKind.Run, (rqst, token)=>void{}, false);
			profile.onDidChangeDefault((newValue) => {
				vscode.window.showInformationMessage("profile.isDefault is: "+newValue);
			});
			vscode.window.showInformationMessage(`SUCCESS created TestRunProfile: ${profile.label}`);
			
			const profile2 = ctrl.createRunProfile("myRunProfile2", vscode.TestRunProfileKind.Run, (rqst, token)=>void{}, false);
			profile2.onDidChangeDefault((newValue) => {
				vscode.window.showInformationMessage("profile2.isDefault is: "+newValue);
			});
			vscode.window.showInformationMessage(`SUCCESS created 2nd TestRunProfile: ${profile2.label}`);

		}catch (error) { 
			vscode.window.showInformationMessage(`FAIL Could not create TestRunProfile`)
			handleError(error);
			throw new Error("Tests failed");
		}

		try { 
			const testItem = ctrl.createTestItem("testItemId", "testItemLabel");
			ctrl.items.add(testItem);

			const subTestItem = ctrl.createTestItem("subTestItemId", "subTestItemLabel");
			testItem.children.add(subTestItem);
			vscode.window.showInformationMessage(`SUCCESS created TestItem: ${testItem.id}`);
		}catch (error) { 
			vscode.window.showInformationMessage(`FAIL Could not create TestItem`)
			handleError(error);
			throw new Error("Tests failed");
		}

		try { 
			const testRun = ctrl.createTestRun(rqst, "testRunName");
			vscode.window.showInformationMessage(`SUCCESS created TestRun: ${testRun.name}`);
			setTimeout(() => testRun.end(), 2000);
		}catch (error) { 
			vscode.window.showInformationMessage(`FAIL Could not create TestRun`);
			handleError(error);
			throw new Error("Tests failed");
		}
	});
	context.subscriptions.push(commandCreateController);
}

// This method is called when your extension is deactivated
export function deactivate() {}
